import java.awt.Color;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import javax.swing.JFrame;

/**
 *
 * @author Boss
 */
public final class Board{

    private static int breiteMin = 900;
    private static int hoeheMin = 640;
    private static int breite = breiteMin;
    private static int hoehe = hoeheMin;
    private static int anzahlLampen = 28;
    private static int anzahlButton = 10;
    private static Board board;
    private static Led[] listLeds = new Led[anzahlLampen];
    private static Button[] listButtons = new Button[anzahlButton];
    private static View view;
    private static long startZeit; // Startzeit fr Stoppuhr speichern
    private static ArrayList<LoopThread> loopThreadListe = new ArrayList<LoopThread>(); // Thread
    private static int indexAktuellerThread = -1;
    private static int indexSchalterGedrueckt = -1;

    /**
     * @param args the command line arguments
     */
    //    public static void main(String[] args) {
    //        init();
    //    }

    public Board() {
        init();
    }

    private static void init() throws HeadlessException {
        JFrame frame = new JFrame();
        frame.setSize(breite, hoehe);
        frame.show();
        frame.setResizable(false);
        view = new View();
        view.setPreferredSize(new Dimension(breite, hoehe));
        view.setBackground(new Color(225, 225, 225));
        frame.add(view);

        erzeugeLeds();
        erzeugeButtons();
        view.setListLeds(listLeds);
        view.setListButtons(listButtons);
        view.repaint();
    }

    public static void setSize(int width, int height){
        breite = width;
        hoehe = height;
        if (breite < breiteMin){
            breite = breiteMin;
        }
        if (hoehe < hoeheMin){
            hoehe = hoeheMin;
        }
    }

    public Button getButton(int index){
        return listButtons[index];
    }

    public Led[] getListLeds() {
        return listLeds;
    }

    public Led getLed(int index) {
        return listLeds[index];
    }

    public void setLed(int index) {
        this.listLeds[index] = new Led(index, Color.red);
    }

    private static void erzeugeLeds(){
        for (int i = 0; i < anzahlLampen; i++){
            listLeds[i] = new Led(i);
        }
    }

    private static void erzeugeButtons(){
        for (int i = 0; i < anzahlButton; i++){
            listButtons[i] = new Button(i);
        }
    }

    // Methoden
    /**
     * Getter fr Singleton
     */
    public static synchronized Board getBoard() {
        if (board == null) {
            board = new Board();
            //     init();
        }
        return board;
    }

    public void turnLedOn(int index){
        listLeds[index].on();
        view.repaint();
    }

    public void turnLedOff(int index){
        listLeds[index].out();
        view.repaint();
    }

    public void pressButtonAsSwitch(int index){
        listButtons[index].stateChanged();
        view.repaint();
    }

    public void pressButtonAsTaste(int index, int repeat){
        for (int i = 0; i < repeat; i++){
            listButtons[index].close();
            view.repaint();
            warten(250);
            listButtons[index].open();
            view.repaint();
            if (i < repeat-1){
                warten(250);
            }
        }
    }

    public boolean istButtonOpen(int index){
        return listButtons[index].isOpen();
    }

    public int getIndexButtonPressed(){
        return view.getIndexButton();
    }

    public static void updateButtonPressed(int index){
        indexSchalterGedrueckt = index;
        //System.out.println("update B" + (indexSchalterGedrueckt+1) + " pressed");
    }

    /**
     * startet die angegebene Methode (ohne Klammern und ohne Parameter!) als fortlaufende Wiederholung
     * @param objekt das aktuelle Objekt
     * @param methodenname der zu startende Methodenname
     */
    public static void loop(Object objekt, String methodenname) {
        loopWithDelay(objekt, methodenname, 0, false); // 0 ms Pause
    }

    /**
     * startet die angegebene Methode (ohne Klammern und ohne Parameter!) als einmalige Wiederholung
     * @param objekt das aktuelle Objekt
     * @param methodenname der zu startende Methodenname
     */
    public static void run(Object objekt, String methodenname) {
        loopWithDelay(objekt, methodenname, 0, true); // 0 ms Pause
    }

    /**
     * startet die angegebene Methode (ohne Klammern und ohne Parameter!) als fortlaufende Wiederholung und wartet nach jeder Ausfhrung fr die angegebene Zeit in Millisekunden
     * @param objekt das aktuelle Objekt
     * @param methodenname der zu startende Methodenname
     * @param millisekunden wartet nach Ausfhrung der Methode die angegebene Zeit in Millisekunden
     */
    public static void loopWithDelay(Object objekt, String methodenname, int millisekunden, boolean status) {
        Method methode=null;
        try {
            //Class klasse=Class.forName(Thread.currentThread().getStackTrace()[2].getClassName());
            //methode=klasse.getMethod(methodenname);        
            methode=objekt.getClass().getMethod(methodenname);        
        }
        catch (Exception e) {
        }
        if (methode!=null) {
            LoopThread st=new LoopThread(objekt, methode, methodenname, millisekunden, status);
            st.start();
            if (!status){
                loopThreadListe.add(st);
            }
        }
        else {
            System.out.println("Methode "+methodenname+" nicht vorhanden");
        }
    }

    /**
     * stoppt die Ausfhrung der fortlaufenden Wiederholung der Methode mit dem angegebenen Index
     * @param index Index der laufenden Methode
     */
    public static void stopLoop(int index) {
        if (index<loopThreadListe.size() && index>=0) {
            loopThreadListe.get(index).stopp();
            loopThreadListe.remove(index);
        }
        else {
            System.out.println("falsche Nummer eingegeben");
        }
    }    

    /**
     * stoppt die Ausfhrung der Wiederholung der Methode 
     * ab dem angegebenen Index
     * @param index Index
     */
    public static void stopp(int ab) {
        for (int i = loopThreadListe.size()-1; i >= ab; i--){
            stopLoop(i);
        }
    }    

    /**
     * stoppt die Ausfhrung der fortlaufenden Wiederholung der Methode mit dem angegebenen Methodennamen
     * @param methodenname Name der laufenden Methode
     */    
    public static void stopLoop(String methodenname) {
        int index=-1; // -1, falls Thread nicht existiert
        if (loopThreadListe.size()>0) {
            for (int i=0; i<loopThreadListe.size(); i=i+1) {
                if (loopThreadListe.get(i).getMethodenname().equals(methodenname)) {
                    index=i;
                }
            }    
        }
        if (index!=-1) {
            loopThreadListe.get(index).stopp();
            loopThreadListe.remove(index);           
        }
        else {
            System.out.println("keine laufende Methode mit dem Namen "+methodenname+" vorhanden");
        }
    }

    /**
     * Listet alle laufenden Threads auf, die mit loop(...) gestartet wurden
     */
    public static void listLoops() {
        if (loopThreadListe.size()>0) {
            System.out.println("Nr.\tName:");
            System.out.println("-------------------------------------");
            for (int i=0; i<loopThreadListe.size(); i=i+1) {
                System.out.println(i+"\t"+loopThreadListe.get(i).getMethodenname());
            }
        }
        else {
            System.out.println("keine laufenden Methoden vorhanden");
        }
    }

    /**
     * startet die Stoppuhr
     */
    public static void startClock() {
        startZeit=System.nanoTime();
    }

    /**
     * stopt die Stoppuhr und gibt verstrichene Zeit seit dem Stoppuhr-Start in Millisekunden zurck
     */
    public static long stopClock() {
        if (startZeit!=-1) {
            return (System.nanoTime()-startZeit)/1000000;
        }
        else {
            System.out.println("die Stoppuhr wurde nicht gestartet!");
            return 0;
        }
    }

    public static int getIndexSchalterGedrueckt(){
        return indexSchalterGedrueckt;
    }

    private void warten(long ms){
        long t1 = System.currentTimeMillis();
        long t2 = t1;
        while (t2 - t1 < ms){
            t2 = System.currentTimeMillis();
        }
    }

}